BV

Struct BV 

pub struct BV { /* private fields */ }
Ast node representing a bitvector value.

Implementations

Source

impl BV

pub fn from_str(sz: u32, value: &str) -> Option<BV>

pub fn from_bits(bits: &[bool]) -> Option<BV>

Create a BV from an array of bits.

Examples
// 0b00000010
let bv = BV::from_bits(&[false, true, false, false, false, false, false, false]).unwrap();
let bv_none = BV::from_bits(&[]);
assert_eq!(bv, 2);
assert_eq!(bv_none, None);
pub fn new_const<S: Into<Symbol>>(name: S, sz: u32) -> BV

pub fn fresh_const(prefix: &str, sz: u32) -> BV

pub fn from_i64(i: i64, sz: u32) -> BV

pub fn from_u64(u: u64, sz: u32) -> BV

pub fn as_i64(&self) -> Option<i64>

pub fn as_u64(&self) -> Option<u64>

pub fn from_int(ast: &Int, sz: u32) -> BV

Create a bit vector from an integer.

The bit vector will have width sz.

Examples
let i = ast::Int::new_const("x");
solver.assert(&i.eq(&ast::Int::from_i64(-3)));

let x = ast::BV::from_int(&i, 64);
assert_eq!(64, x.get_size());

assert_eq!(solver.check(), SatResult::Sat);
let model = solver.get_model().unwrap();

assert_eq!(-3, model.eval(&x.to_int(true), true).unwrap().as_i64().expect("as_i64() shouldn't fail"));
pub fn to_int(&self, signed: bool) -> Int

Create an integer from a bitvector. This is just a convenience wrapper around Int::from_bv(); see notes there.

pub fn get_size(&self) -> u32

Get the size of the bitvector (in bits)

pub fn bvnot(&self) -> Self

Bitwise negation

pub fn bvneg(&self) -> Self

Two’s complement negation

pub fn bvredand(&self) -> Self

Conjunction of all the bits in the vector. Returns a BV with size (bitwidth) 1.

pub fn bvredor(&self) -> Self

Disjunction of all the bits in the vector. Returns a BV with size (bitwidth) 1.

pub fn bvand<T: IntoAst<Self>>(&self, other: T) -> Self

Bitwise and

pub fn bvor<T: IntoAst<Self>>(&self, other: T) -> Self

Bitwise or

pub fn bvxor<T: IntoAst<Self>>(&self, other: T) -> Self

Bitwise exclusive-or

pub fn bvnand<T: IntoAst<Self>>(&self, other: T) -> Self

Bitwise nand

pub fn bvnor<T: IntoAst<Self>>(&self, other: T) -> Self

Bitwise nor

pub fn bvxnor<T: IntoAst<Self>>(&self, other: T) -> Self

Bitwise xnor

pub fn bvadd<T: IntoAst<Self>>(&self, other: T) -> Self

Addition

pub fn bvsub<T: IntoAst<Self>>(&self, other: T) -> Self

Subtraction

pub fn bvmul<T: IntoAst<Self>>(&self, other: T) -> Self

Multiplication

pub fn bvudiv<T: IntoAst<Self>>(&self, other: T) -> Self

Unsigned division

pub fn bvsdiv<T: IntoAst<Self>>(&self, other: T) -> Self

Signed division

pub fn bvurem<T: IntoAst<Self>>(&self, other: T) -> Self

Unsigned remainder

pub fn bvsrem<T: IntoAst<Self>>(&self, other: T) -> Self

Signed remainder (sign follows dividend)

pub fn bvsmod<T: IntoAst<Self>>(&self, other: T) -> Self

Signed remainder (sign follows divisor)

pub fn bvult<T: IntoAst<Self>>(&self, other: T) -> Bool

Unsigned less than

pub fn bvslt<T: IntoAst<Self>>(&self, other: T) -> Bool

Signed less than

pub fn bvule<T: IntoAst<Self>>(&self, other: T) -> Bool

Unsigned less than or equal

pub fn bvsle<T: IntoAst<Self>>(&self, other: T) -> Bool

Signed less than or equal

pub fn bvuge<T: IntoAst<Self>>(&self, other: T) -> Bool

Unsigned greater or equal

pub fn bvsge<T: IntoAst<Self>>(&self, other: T) -> Bool

Signed greater or equal

pub fn bvugt<T: IntoAst<Self>>(&self, other: T) -> Bool

Unsigned greater than

pub fn bvsgt<T: IntoAst<Self>>(&self, other: T) -> Bool

Signed greater than

pub fn bvshl<T: IntoAst<Self>>(&self, other: T) -> Self

Shift left

pub fn bvlshr<T: IntoAst<Self>>(&self, other: T) -> Self

Logical shift right (add zeroes in the high bits)

pub fn bvashr<T: IntoAst<Self>>(&self, other: T) -> Self

Arithmetic shift right (sign-extend in the high bits)

pub fn bvrotl<T: IntoAst<Self>>(&self, other: T) -> Self

Rotate left

pub fn bvrotr<T: IntoAst<Self>>(&self, other: T) -> Self

Rotate right

pub fn concat<T: IntoAst<Self>>(&self, other: T) -> Self

Concatenate two bitvectors

pub fn bvneg_no_overflow(&self) -> Bool

Check if negation overflows

pub fn bvadd_no_overflow(&self, other: &BV, b: bool) -> Bool

Check if addition overflows

pub fn bvsub_no_underflow(&self, other: &BV, b: bool) -> Bool

Check if subtraction underflows

pub fn bvmul_no_overflow(&self, other: &BV, b: bool) -> Bool

Check if multiplication overflows

pub fn bvadd_no_underflow<T: IntoAst<Self>>(&self, other: T) -> Bool

Check if addition underflows

pub fn bvsub_no_overflow<T: IntoAst<Self>>(&self, other: T) -> Bool

Check if subtraction overflows

pub fn bvsdiv_no_overflow<T: IntoAst<Self>>(&self, other: T) -> Bool

Check if signed division overflows

pub fn bvmul_no_underflow<T: IntoAst<Self>>(&self, other: T) -> Bool

Check if multiplication underflows

pub fn extract(&self, high: u32, low: u32) -> Self

Extract the bits high down to low from the bitvector. Returns a bitvector of size n, where n = high - low + 1.

pub fn sign_ext(&self, i: u32) -> Self

Sign-extend the bitvector to size m+i, where m is the original size of the bitvector. That is, i bits will be added.

pub fn zero_ext(&self, i: u32) -> Self

Zero-extend the bitvector to size m+i, where m is the original size of the bitvector. That is, i bits will be added.

Source

impl BV

pub fn ast_eq<T: IntoAst<Self>>(&self, other: T) -> bool
where Self: Sized,

pub fn _eq<T: IntoAst<Self>>(&self, other: T) -> Bool
where Self: Sized,

👎Deprecated:

Please use eq instead

pub fn ne<T: IntoAst<Self>>(&self, other: T) -> Bool
where Self: Sized,

pub fn eq<T: IntoAst<Self>>(&self, other: T) -> Bool
where Self: Sized,

Compare this Ast with another Ast, and get a Bool representing the result.

This operation works with all possible Asts (int, real, BV, etc), but the two Asts being compared must be the same type.

pub fn _safe_eq<T: IntoAst<Self>>(&self, other: T) -> Result<Bool, SortDiffers>
where Self: Sized,

👎Deprecated:

Please use safe_eq instead

pub fn safe_eq<T: IntoAst<Self>>(&self, other: T) -> Result<Bool, SortDiffers>
where Self: Sized,

Compare this Ast with another Ast, and get a Result. Errors if the sort does not match for the two values.