Bool

Struct Bool 

pub struct Bool { /* private fields */ }
Ast node representing a boolean value.

Implementations

Source

impl Bool

pub fn new_const<S: Into<Symbol>>(name: S) -> Bool

pub fn fresh_const(prefix: &str) -> Bool

pub fn from_bool(b: bool) -> Bool

pub fn as_bool(&self) -> Option<bool>

pub fn ite<T>(&self, a: &T, b: &T) -> T
where T: Ast,

pub fn and<T: Into<Self> + Clone>(values: &[T]) -> Self

pub fn or<T: Into<Self> + Clone>(values: &[T]) -> Self

pub fn xor<T: IntoAst<Self>>(&self, other: T) -> Self

pub fn iff<T: IntoAst<Self>>(&self, other: T) -> Self

pub fn implies<T: IntoAst<Self>>(&self, other: T) -> Self

pub fn not(&self) -> Self

pub fn pb_le(values: &[(&Bool, i32)], k: i32) -> Bool

pub fn pb_ge(values: &[(&Bool, i32)], k: i32) -> Bool

pub fn pb_eq(values: &[(&Bool, i32)], k: i32) -> Bool

Source

impl Bool

pub fn ast_eq<T: IntoAst<Self>>(&self, other: T) -> bool
where Self: Sized,

pub fn _eq<T: IntoAst<Self>>(&self, other: T) -> Bool
where Self: Sized,

👎Deprecated: Please use eq instead
pub fn ne<T: IntoAst<Self>>(&self, other: T) -> Bool
where Self: Sized,

pub fn eq<T: IntoAst<Self>>(&self, other: T) -> Bool
where Self: Sized,

Compare this Ast with another Ast, and get a Bool representing the result.

This operation works with all possible Asts (int, real, BV, etc), but the two Asts being compared must be the same type.

pub fn _safe_eq<T: IntoAst<Self>>(&self, other: T) -> Result<Bool, SortDiffers>
where Self: Sized,

👎Deprecated: Please use safe_eq instead
pub fn safe_eq<T: IntoAst<Self>>(&self, other: T) -> Result<Bool, SortDiffers>
where Self: Sized,

Compare this Ast with another Ast, and get a Result. Errors if the sort does not match for the two values.