Dynamic

Struct Dynamic 

pub struct Dynamic { /* private fields */ }
A dynamically typed Ast node.

Implementations

Source

impl Dynamic

pub fn from_ast(ast: &dyn Ast) -> Self

pub fn new_const<S: Into<Symbol>>(name: S, sort: &Sort) -> Self

pub fn fresh_const(prefix: &str, sort: &Sort) -> Self

pub fn sort_kind(&self) -> SortKind

pub fn as_bool(&self) -> Option<Bool>

Returns None if the Dynamic is not actually a Bool

pub fn as_int(&self) -> Option<Int>

Returns None if the Dynamic is not actually an Int

pub fn as_real(&self) -> Option<Real>

Returns None if the Dynamic is not actually a Real

pub fn as_float(&self) -> Option<Float>

Returns None if the Dynamic is not actually a Float

pub fn as_string(&self) -> Option<String>

Returns None if the Dynamic is not actually a String

pub fn as_bv(&self) -> Option<BV>

Returns None if the Dynamic is not actually a BV

pub fn as_array(&self) -> Option<Array>

Returns None if the Dynamic is not actually an Array

pub fn as_set(&self) -> Option<Set>

Returns None if the Dynamic is not actually a Set

pub fn as_seq(&self) -> Option<Seq>

Returns None if the Dynamic is not actually a Seq.

pub fn as_datatype(&self) -> Option<Datatype>

Returns None if the Dynamic is not actually a Datatype

Source

impl Dynamic

pub fn ast_eq<T: IntoAst<Self>>(&self, other: T) -> bool
where Self: Sized,

pub fn _eq<T: IntoAst<Self>>(&self, other: T) -> Bool
where Self: Sized,

👎Deprecated: Please use eq instead
pub fn ne<T: IntoAst<Self>>(&self, other: T) -> Bool
where Self: Sized,

pub fn eq<T: IntoAst<Self>>(&self, other: T) -> Bool
where Self: Sized,

Compare this Ast with another Ast, and get a Bool representing the result.

This operation works with all possible Asts (int, real, BV, etc), but the two Asts being compared must be the same type.

pub fn _safe_eq<T: IntoAst<Self>>(&self, other: T) -> Result<Bool, SortDiffers>
where Self: Sized,

👎Deprecated: Please use safe_eq instead
pub fn safe_eq<T: IntoAst<Self>>(&self, other: T) -> Result<Bool, SortDiffers>
where Self: Sized,

Compare this Ast with another Ast, and get a Result. Errors if the sort does not match for the two values.