RoundingMode

Struct RoundingMode 

pub struct RoundingMode { /* private fields */ }
Ast node representing a rounding mode for Float operations.

Implementations

Source

impl RoundingMode

pub fn round_towards_zero() -> RoundingMode

Create a numeral of RoundingMode sort which represents the TowardZero rounding mode.

pub fn round_towards_negative() -> RoundingMode

Create a numeral of RoundingMode sort which represents the TowardNegative rounding mode.

pub fn round_towards_positive() -> RoundingMode

Create a numeral of RoundingMode sort which represents the TowardPositive rounding mode.

pub fn round_nearest_ties_to_away() -> RoundingMode

Create a numeral of RoundingMode sort which represents the NearestTiesToAway rounding mode.

pub fn round_nearest_ties_to_even() -> RoundingMode

Create a numeral of RoundingMode sort which represents the NearestTiesToEven rounding mode.

pub fn add<A: Into<Float>, B: IntoAst<Float>>(&self, a: A, b: B) -> Float

pub fn sub<A: Into<Float>, B: IntoAst<Float>>(&self, a: A, b: B) -> Float

pub fn mul<A: Into<Float>, B: IntoAst<Float>>(&self, a: A, b: B) -> Float

pub fn div<A: Into<Float>, B: IntoAst<Float>>(&self, a: A, b: B) -> Float

Source

impl RoundingMode

pub fn ast_eq<T: IntoAst<Self>>(&self, other: T) -> bool
where Self: Sized,

pub fn _eq<T: IntoAst<Self>>(&self, other: T) -> Bool
where Self: Sized,

👎Deprecated: Please use eq instead
pub fn ne<T: IntoAst<Self>>(&self, other: T) -> Bool
where Self: Sized,

pub fn eq<T: IntoAst<Self>>(&self, other: T) -> Bool
where Self: Sized,

Compare this Ast with another Ast, and get a Bool representing the result.

This operation works with all possible Asts (int, real, BV, etc), but the two Asts being compared must be the same type.

pub fn _safe_eq<T: IntoAst<Self>>(&self, other: T) -> Result<Bool, SortDiffers>
where Self: Sized,

👎Deprecated: Please use safe_eq instead
pub fn safe_eq<T: IntoAst<Self>>(&self, other: T) -> Result<Bool, SortDiffers>
where Self: Sized,

Compare this Ast with another Ast, and get a Result. Errors if the sort does not match for the two values.