AstVector

Struct AstVector 

pub struct AstVector { /* private fields */ }
Vector of Z3 AST nodes.

This wraps a Z3-ref-counted vector of asts. This type exists mainly as a high-level compatibility layer for these types from Rust. As this type wraps heterogenous asts, it is analagous to Vec<Dynamic>.

Most users will usually be better off using a Vec; this type primarily exists as a compatibility shim.

Many standard rust collection and iteration traits are implemented on it for convenience.

Implementations

Source

impl AstVector

pub fn new() -> AstVector

Create a new empty AST vector.

pub fn len(&self) -> usize

Get the number of elements in the vector.

pub fn is_empty(&self) -> bool

Check if the vector is empty.

pub fn get(&self, index: usize) -> Dynamic

Get the element at the specified index.

Panics

Panics if the index is out of bounds.

pub fn set(&self, index: usize, ast: &impl Ast)

Set the element at the specified index.

Panics

Panics if the index is out of bounds.

pub fn push(&self, ast: &impl Ast)

Push an element to the end of the vector.

pub fn resize(&self, new_size: usize)

Resize the vector to the specified size.

New elements (if any) are uninitialized.

pub fn to_vec(&self) -> Vec<Dynamic>

Convert the vector to a Vec<Dynamic>.

pub fn translate(&self, target_ctx: &Context) -> AstVector

Translate the AST vector to another context.

pub fn iter(&self) -> AstVectorIter<'_>

Return a borrowing iterator over the elements.