Model

Struct Model 

pub struct Model { /* private fields */ }
Model for the constraints inserted into the logical context.

Implementations

Source

impl Model

pub fn of_solver(slv: &Solver) -> Option<Model>

pub fn of_optimize(opt: &Optimize) -> Option<Model>

pub fn get_const_interp<T: Ast>(&self, ast: &T) -> Option<T>

Returns the interpretation of the given ast in the Model Returns None if there is no interpretation in the Model

pub fn get_func_interp(&self, f: &FuncDecl) -> Option<FuncInterp>

Returns the interpretation of the given f in the Model Returns None if there is no interpretation in the Model

pub fn eval<T>(&self, ast: &T, model_completion: bool) -> Option<T>
where T: Ast,

pub fn iter<'a>(&'a self) -> ModelIter<'a>