pub struct Dynamic { /* private fields */ }Expand description
A dynamically typed Ast node.
Implementations
Sourceimpl Dynamic
impl Dynamic
pub fn from_ast(ast: &dyn Ast) -> Self
pub fn new_const<S: Into<Symbol>>(name: S, sort: &Sort) -> Self
pub fn fresh_const(prefix: &str, sort: &Sort) -> Self
pub fn sort_kind(&self) -> SortKind
Sourcepub fn as_datatype(&self) -> Option<Datatype>
pub fn as_datatype(&self) -> Option<Datatype>
Returns None if the Dynamic is not actually a Datatype
Sourceimpl Dynamic
impl Dynamic
pub fn ast_eq<T: IntoAst<Self>>(&self, other: T) -> boolwhere
Self: Sized,
pub fn _eq<T: IntoAst<Self>>(&self, other: T) -> Boolwhere
Self: Sized,
👎Deprecated: Please use eq instead
pub fn ne<T: IntoAst<Self>>(&self, other: T) -> Boolwhere
Self: Sized,
Sourcepub fn eq<T: IntoAst<Self>>(&self, other: T) -> Boolwhere
Self: Sized,
pub fn eq<T: IntoAst<Self>>(&self, other: T) -> Boolwhere
Self: Sized,
Compare this Ast with another Ast, and get a Bool
representing the result.
This operation works with all possible Asts (int, real, BV, etc), but the two
Asts being compared must be the same type.
pub fn _safe_eq<T: IntoAst<Self>>(&self, other: T) -> Result<Bool, SortDiffers>where
Self: Sized,
👎Deprecated: Please use safe_eq instead