pub struct AstVector { /* private fields */ }Expand description
Vector of Z3 AST nodes.
This wraps a Z3-ref-counted vector of asts. This type exists
mainly as a high-level compatibility layer for these types from Rust.
As this type wraps heterogenous asts, it is analagous to Vec<Dynamic>.
Most users will usually be better off using a Vec; this type primarily exists as a compatibility shim.
Many standard rust collection and iteration traits are implemented on it for convenience.
Implementations
Sourceimpl AstVector
impl AstVector
Sourcepub fn resize(&self, new_size: usize)
pub fn resize(&self, new_size: usize)
Resize the vector to the specified size.
New elements (if any) are uninitialized.
Sourcepub fn translate(&self, target_ctx: &Context) -> AstVector
pub fn translate(&self, target_ctx: &Context) -> AstVector
Translate the AST vector to another context.
Sourcepub fn iter(&self) -> AstVectorIter<'_> ⓘ
pub fn iter(&self) -> AstVectorIter<'_> ⓘ
Return a borrowing iterator over the elements.